package com.example.demo.core.security;

import java.util.Collection;
import java.util.Set;

import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import com.example.demo.users.model.UserEntity;

// Класс для аутентификации пользователя
public class UserPrincipal implements UserDetails {
    // Идентификатор
    private final long id;

    // Имя/логин пользователя
    private final String username;

    // Пароль пользователя
    private final String password;

    // Список ролей пользователя
    private final Set<? extends GrantedAuthority> roles;

    // Активный аккаунт
    private final boolean active;

    public UserPrincipal(UserEntity user) {
        this.id = user.getId();
        this.username = user.getUsername();
        this.password = user.getPassword();
        this.roles = Set.of(user.getRole());
        this.active = true;
    }
 
    // Получить идентификатор
    public Long getId() {
        return id;
    }

    // Получить имя/логин пользователя
    @Override
    public String getUsername() {
        return username;
    }

    // Получить пароль пользователя
    @Override
    public String getPassword() {
        return password;
    }

    // Получить список ролей пользователя
    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return roles;
    }

    // Получить признак активности пользователя
    @Override
    public boolean isEnabled() {
        return active;
    }

    // Признак актуальности учетной записи пользователя
    @Override
    public boolean isAccountNonExpired() {
        return isEnabled();
    }

    // Признак блокировки пользователя
    @Override
    public boolean isAccountNonLocked() {
        return isEnabled();
    }

    // Признак актуальности учетной записи пользователя
    @Override
    public boolean isCredentialsNonExpired() {
        return isEnabled();
    }
}
