package com.example.demo.users.api;

import java.time.LocalDateTime;

import org.modelmapper.ModelMapper;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.demo.core.configuration.Constants;
import com.example.demo.core.security.UserPrincipal;
import com.example.demo.messages.api.MessageDto;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;

import jakarta.validation.Valid;

// Контроллер для работы с корзиной покупок пользователя
@Controller
@RequestMapping(UserMessageController.URL)
public class UserMessageController {
    // URL для доступа к методам контроллера
    public static final String URL = "/message";

    // Представление для написания сообщения
    private static final String MESSAGE_EDIT_VIEW = "message-edit";

    // Атрибут модели для обработки данных
    private static final String MESSAGE_ATTRIBUTE = "message";

    // Атрибут модели для пагинации
    private static final String PAGE_ATTRIBUTE = "page";

    // Бизнес-логика для сущости "Сообщение"
    private final MessageService messageService;

    // Библиотека для преобразования сущности
    private final ModelMapper modelMapper;

    public UserMessageController(MessageService messageService, ModelMapper modelMapper) {
        this.messageService = messageService;
        this.modelMapper = modelMapper;
    }

    private MessageEntity toEntity(MessageDto dto) {
        return modelMapper.map(dto, MessageEntity.class);
    }

    // Отправить сообщение
    @GetMapping("/send/")
    public String sendMessage(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        model.addAttribute(MESSAGE_ATTRIBUTE, new MessageDto());
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return MESSAGE_EDIT_VIEW;
    }

    // Отправить сообщение
    @PostMapping("/send/")
    public String sendMessage(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = MESSAGE_ATTRIBUTE) @Valid MessageDto message,
            BindingResult bindingResult,
            Model model,
            @AuthenticationPrincipal UserPrincipal principal,
            RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return MESSAGE_EDIT_VIEW;
        }

        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        message.setDate(LocalDateTime.now());
        messageService.create(principal.getId(), toEntity(message));
        return Constants.REDIRECT_VIEW + "/";
    }
}
