package com.example.demo.users.model;

import java.util.Objects;
import java.util.Set;
import java.util.HashSet;

import com.example.demo.core.model.BaseEntity;
import com.example.demo.orders.model.OrderEntity;
import com.example.demo.messages.model.MessageEntity;

import jakarta.persistence.Entity;
import jakarta.persistence.Table;
import jakarta.persistence.Column;
import jakarta.persistence.OneToMany;
import jakarta.persistence.CascadeType;
import jakarta.persistence.OrderBy;

// Сущность "Пользователь"
@Entity
@Table(name = "users")
public class UserEntity extends BaseEntity {
    // Имя/логин пользователя
    @Column(nullable = false, unique = true, length = 50)
    private String username;

    // Пароль пользователя
    @Column(nullable = false, length = 60)
    private String password;

    // Электронный адрес почты пользователя
    @Column(nullable = false, unique = true)
    private String email;

    // Роль пользователя
    private UserRole role;

    // Список заказов пользователя
    @OneToMany(mappedBy = "user", cascade = CascadeType.ALL)
    @OrderBy("id ASC")
    private Set<OrderEntity> orders = new HashSet<>();

    // Список сообщений пользователя
    @OneToMany(mappedBy = "user", cascade = CascadeType.ALL)
    @OrderBy("id ASC")
    private Set<MessageEntity> messages = new HashSet<>();

    // Конструктор по умолчанию
    public UserEntity() {
    }

    // Конструктор с параметрами для создания объекта
    public UserEntity(String username, String password, String email) {
        this.username = username;
        this.password = password;
        this.email = email;
        this.role = UserRole.USER;
    }

    // Получить имя/логин пользователя
    public String getUsername() {
        return username;
    }

    // Установить имя/логин пользователя
    public void setUsername(String username) {
        this.username = username;
    }

    // Получить пароль пользователя
    public String getPassword() {
        return password;
    }

    // Установить пароль пользователя
    public void setPassword(String password) {
        this.password = password;
    }

    // Получить электронный адрес почты пользователя
    public String getEmail() {
        return email;
    }

    // Установить электронный адрес почты пользователя
    public void setEmail(String email) {
        this.email = email;
    }

    // Получить роль пользователя
    public UserRole getRole() {
        return role;
    }

    // Установить роль пользователя
    public void setRole(UserRole role) {
        this.role = role;
    }

    // Получить список заказов
    public Set<OrderEntity> getOrders() {
        return orders;
    }

    // Добавить заказ
    public void addOrder(OrderEntity order) {
        if (order.getUser() != this) {
            order.setUser(this);
        }
        orders.add(order);
    }

    // Получить список сообщений
    public Set<MessageEntity> getMessages() {
        return messages;
    }

    // Добавить сообщение
    public void addMessage(MessageEntity message) {
        if (message.getUser() != this) {
            message.setUser(this);
        }
        messages.add(message);
    }

    // Получить хэш-код объекта
    @Override
    public int hashCode() {
        return Objects.hash(id, username, password, email, role, orders, messages);
    }

    // Сравнить объекты
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null || getClass() != obj.getClass())
            return false;
        final UserEntity other = (UserEntity) obj;
        return Objects.equals(other.getId(), id)
                && Objects.equals(other.getUsername(), username)
                && Objects.equals(other.getPassword(), password)
                && Objects.equals(other.getEmail(), email)
                && Objects.equals(other.getRole(), role)
                && Objects.equals(other.getOrders(), orders)
                && Objects.equals(other.getMessages(), messages);
    }
}
