package com.example.demo;

import java.util.List;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;

import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.orders.model.OrderEntity;
import com.example.demo.orders.service.OrderService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

import jakarta.persistence.EntityManager;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class UserOrderServiceTests {
    @Autowired
    private EntityManager entityManager;

    @Autowired
    private TypeService typeService;

    @Autowired
    private UserService userService;

    @Autowired
    private OrderService orderService;

    private TypeEntity type1;
    private TypeEntity type2;
    private TypeEntity type3;

    private UserEntity user1;
    private UserEntity user2;

    @BeforeEach
    void createData() {
        removeData();

        type1 = typeService.create(new TypeEntity("Protection"));
        type2 = typeService.create(new TypeEntity("Sharpness"));
        type3 = typeService.create(new TypeEntity("Infinity"));

        user1 = userService.create(new UserEntity("user1", "password", "mail1@gmail.com"));
        user2 = userService.create(new UserEntity("user2", "password", "mail2@gmail.com"));

        final var orders = List.of(
            new OrderEntity(type1, 12.00, 3),
			new OrderEntity(type1, 50.00, 20),
			new OrderEntity(type2, 15.00, 30),
			new OrderEntity(type2, 64.00, 10),
			new OrderEntity(type2, 15.00, 6),
			new OrderEntity(type3, 80.00, 6),
			new OrderEntity(type3, 64.00, 3)
        );
        orders.forEach(order -> orderService.create(user1.getId(), order));
    }

    @AfterEach
    void removeData() {
        userService.deleteAll();
        typeService.deleteAll();
    }

    @Test
    @Order(1)
    void createTest() {
        Assertions.assertEquals(7, orderService.getAll(user1.getId(), 0L).size());
        Assertions.assertEquals(0, orderService.getAll(user2.getId(), 0L).size());
    }

    @Test
    @Order(2)
    void orderFilterTest() {
        Assertions.assertEquals(2, orderService.getAll(user1.getId(), type1.getId()).size());
        Assertions.assertEquals(3, orderService.getAll(user1.getId(), type2.getId()).size());
        Assertions.assertEquals(2, orderService.getAll(user1.getId(), type3.getId()).size());
    }

    @Test
    @Order(3)
    void userCascadeDeleteTest() {
        userService.delete(user1.getId());
        final var orders = entityManager.createQuery(
                "select count(o) from OrderEntity o where o.user.id = :userId");
        orders.setParameter("userId", user1.getId());
        Assertions.assertEquals(0, orders.getFirstResult());
    }
}
